clear; clc; clf
x = linspace(-50,50,100);
y = x;

observation_points = [-40 30;
                       40 30;
                       -30 -40;
                       30 -40];
                         

for i = 1:numel(x)
    for j = 1:numel(y)
        point = [x(i) y(j)];
        objfun(j,i) = f(point);
    end
end

% 3D plot of the objective function
figure(1)
surf(x,y,objfun); title('f(x,y) = 0.5*||h(x,y)||^2'); xlabel('x'); ylabel('y'); zlabel('f(x,y)'); hold on;

% Contour plot of objective function and observation points (blue stars)
figure(2)
contour(x,y,objfun, 30); grid on; hold on; xlabel('x'); ylabel('y'); title('Gauss-Newton method - click for next iterate!');
plot(observation_points(:,1), observation_points(:,2), 'black*'); hold on;

x = [45; 45];                        % Initial point
plot(x(1), x(2), 'ro'); hold on;    % Plot initial point (red ring)
nrIter = 6;

for i = 1:nrIter
    gradient = gradh(x);
    waitforbuttonpress;
    % Equation 17.5 in the book
    direction = ( (gradient)'*gradient )\( -(gradient)'*h(x) );
    
    x = x + direction;
    plot(x(1), x(2), 'r*'); hold on;
end
